from __future__ import unicode_literals

import string
import itertools
import click
import xlrd
import os

from . import asciitable, __version__
from .compat import open_file, csv, out_proc
from .cursor import XCursor, CSVCursor
from six import PY3
from subprocess import Popen, PIPE


@click.command()
@click.option('--heading',
              '-h',
              type=int,
              help='Row number containing the headings.')
@click.option("--file-type", "-f", type=click.Choice(["csv", "excel"]),
              help="Force parsing of the file to the chosen format.")
@click.option("--delimiter", "-d", type=str, default=",",
              help="Delimiter (only applicable to CSV files) [default: ',']")
@click.option("--quotechar", "-q", type=str, default='"',
              help="Quote character (only applicable to CSV files) [default: '\"']")
@click.option("--encoding", "-e", type=str, default="utf-8",
              help="Encoding [default: UTF-8]")
@click.version_option(version=__version__)
@click.argument('filename')
def cli(filename, heading, file_type, delimiter, quotechar, encoding):
    """Display Excel or CSV files directly on your terminal.
    The file type is guessed from file extensions, but can be overridden with the --file-type option.
    """
    if file_type is None:
        if filename.endswith(".csv"):
            file_type = "csv"
        else:
            file_type = "excel"

    if file_type == "csv":

        csv_rows = []

        if not PY3:
            delimiter = str(unicode(delimiter))
            quotechar = str(unicode(quotechar))

        with open_file(filename, encoding) as f:
            reader = csv.reader(f, delimiter=delimiter, quotechar=quotechar)
            for row in reader:
                csv_rows.append(row)

        cursor = CSVCursor(csv_rows, heading)

    else:
        # As per https://secure.simplistix.co.uk/svn/xlrd/trunk/xlrd/doc/xlrd.html?p=4966
        # encodings in Excel are usually UTF-8. So, we only override the encoding
        # if an encoding is specified by the user.
        if encoding.lower() != "utf-8":
            workbook = xlrd.open_workbook(filename, encoding_override=encoding)
        else:
            workbook = xlrd.open_workbook(filename)

        sheet = workbook.sheet_by_index(0)

        cursor = XCursor(sheet, heading)

    with out_proc() as out:

        asciitable.draw(cursor, out=out)



from __future__ import print_function
import codecs
from six import PY3
from contextlib import contextmanager
from subprocess import Popen, PIPE

if PY3:
    import csv
else:
    import unicodecsv as csv


def open_file(filename, encoding):
    """For opening files for reading with a specified encoding (usually UTF-8)"""
    if PY3:
        return open(filename, encoding=encoding)
    else:
        return codecs.open(filename, "r", encoding)


def write_out(s, out, encoding="utf-8"):
    """Provides 2 vs 3 compatibility for writing to ``out``"""
    try:
        if PY3:
            if isinstance(s, bytes):
                out.stdin.write(s)
            else:
                out.stdin.write(bytes(s, encoding))
        else:
            out.stdin.write(s)
    except IOError:
        exit()


@contextmanager
def out_py2():
    """Wrapping our call to ``less`` via ``Popen`` into a basic context manager for Python 2"""
    out = Popen('less -FXRiS', shell=True, bufsize=0, stdin=PIPE)
    yield out
    out.wait()


def out_proc():
    """A context-enabled call to Popen"""
    if PY3:
        return Popen('less -FXRiS', shell=True, bufsize=0, stdin=PIPE)
    else:
        return out_py2()



import itertools
import string


class XCursor(object):

    def __init__(self, sheet, headingrow=None):
        self.headingrow = headingrow
        self.sheet = sheet

    def num_rows(self):
        return self.sheet.nrows

    def num_cols(self):
        if self.headingrow is None:
            return len(self.sheet_row(0))
        else:
            return len(self.sheet_row(self.headingrow))

    def sheet_row(self, n):
        return [c.value for c in self.sheet.row(n)]

    def keys(self):
        if self.headingrow is not None:
            return self.sheet_row(self.headingrow)
        else:
            u = string.ascii_uppercase
            return [''.join(r) for idx, r in
                    enumerate(itertools.chain(u, itertools.product(u, u)))
                    if idx < self.num_cols()]

    def __iter__(self):
        start = self.headingrow + 1 if self.headingrow is not None else 0
        return (self.sheet_row(n) for n in range(start, self.num_rows()))


class CSVCursor(XCursor):

    def sheet_row(self, n):
        return self.sheet[n]

    def num_rows(self):
        return len(self.sheet)
